package com.zfire.jiasm.syncdata.upload.order;

import com.zfire.jiasm.syncdata.constant.GDOrderStatusEnum;
import com.zfire.jiasm.syncdata.constant.OrderTypeEnum;
import com.zfire.jiasm.syncdata.plus.entity.OrderBase;
import com.zfire.jiasm.syncdata.plus.entity.OutJiasmToGreeUploadTaskList;
import com.zfire.jiasm.syncdata.plus.entity.OutJiasmToGreeUploadnotifylist;
import com.zfire.jiasm.syncdata.plus.service.*;
import com.zfire.jiasm.syncdata.service.PrepareService;
import com.zfire.jiasm.syncdata.service.PublicService;
import com.zfire.jiasm.syncdata.utils.PrepareTaskLog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 生成上传任务清单 out_jiasm_to_gree_upload_task_list
 * 1:从需要上传的订单数据表中提取数据 out_jiasm_to_gree_uploadnotifylist
 *:2:对应表中提取表中数据 生成上传任务清单
 */
@Slf4j
@Component
public class RepairProduceUploadList {
    @Resource
    InstallProduceUploadList installProduceUploadList;
    @Resource
    PrepareService prepareService;
    @Resource
    OutJiasmToGreeUploadTaskListService outJiasmToGreeUploadTaskListService;
    @Resource
    OrderBaseService orderBaseService;
    @Resource
    OutJiasmToGreeUploadnotifylistService outJiasmToGreeUploadnotifylistService;
    @Resource
    OrderInstallOperatingLogService orderInstallOperatingLogService;
    @Resource
    OrderReAppointmentService orderReAppointmentService;
    @Resource
    PublicService publicService;


    @Scheduled(fixedDelay = 1 * 20 * 1000)
    private void produceList(){

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<OutJiasmToGreeUploadTaskList> ojtgutls = new ArrayList<>();


        log.info("开始预处理需要上传的维修工单数据: {}",sdf.format(new Date()));

        // 从需要上传的订单数据表中提取数据
        List<OutJiasmToGreeUploadnotifylist> taskResults = outJiasmToGreeUploadnotifylistService.lambdaQuery()
                .eq(OutJiasmToGreeUploadnotifylist::getSynStatus,0)
                .orderByAsc(OutJiasmToGreeUploadnotifylist::getSynReqId).list();
        try{
            for (OutJiasmToGreeUploadnotifylist item : taskResults) {
                Long itemSynReqId = item.getSynReqId();
                // 同步的表名
                String itemTableName = item.getTableName();
                // 同步表中的工单号
                String itemOrderNo = item.getOrderNo();
                // 同步表中的主键id
                String itemTableNameId = item.getId();
                // 工单类型
                String itemOrderType = item.getOrderType();

                // 数据同步预处理规则表中提取对应表名的规则
                Map<String, Object> row = prepareService.getTaskInfo(itemTableName);
                int taskId = (Integer) row.get("id");
                String srcTabName = (String) row.get("src_tab_name");

                String errorMessage = null;
                try{
                    // 安装工单处理
                    if (itemTableName.equals("order_base") &&
                            itemOrderType.equals(OrderTypeEnum.ORDER_TYPE_REPAIR.getCode())){
                        // 过程名，源表中的ID，源表中的数据最后修改时间，源表中的工单号，源表中的id,空值，false
                        // TL_PrePare_task_by_fa_order_repair
                        // 提取工单数据
                        OrderBase orderBase = orderBaseService.lambdaQuery()
                                .eq(OrderBase::getId,itemTableNameId).one();
                        // 生成任务清单数据
                        if (orderBase != null){
                            // 删除工单更新被忽略

                            // 生成66类型上传任务数据
                            // 单号，主键，同步任务生成时间，同步的表名，工单类型，任务类型
                            publicService.getUploadTaskList(itemOrderNo,itemTableNameId,orderBase.getUpdateTime(),itemTableName,
                                    orderBase.getOrderType(),66);

                            // 工单状态(8, 9, 10, 11, 12, 13, 14) 派工状态:(13, 14, 15) 未下载总部采集数据
                            if (orderBase.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G6.getCode()) ||
                                    orderBase.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G7.getCode()) ||
                                    orderBase.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G9.getCode()) ||
                                    orderBase.getOrderStatus().equals(GDOrderStatusEnum.INSTALL_STATUS_G10.getCode())){

                            }






                        }
                    }

                } catch (Exception ex) {
                    errorMessage = ex.getMessage();
                    updateError(errorMessage, item);
                } finally {
                    PrepareTaskLog.writeMessage(genInfoData(taskId, srcTabName, false, errorMessage));
                }
            }



        } catch (Exception ex) {
        ex.printStackTrace();
        log.info("开始预处理需要上传的维修工单数据出错");
    }


    }

    public void updateError(String errMessage, OutJiasmToGreeUploadnotifylist item) {
        if (errMessage != null && errMessage.trim().length() > 255) {
            errMessage = errMessage.substring(0, 255);
        }
        OutJiasmToGreeUploadnotifylist up = new OutJiasmToGreeUploadnotifylist();
        up.setSynReqId(item.getSynReqId());
        up.setSynTimes(item.getSynTimes()+1);
        up.setSynErrMsg(errMessage);
        up.setSynReqTime(new Date());
        outJiasmToGreeUploadnotifylistService.updateById(up);
    }

    private String genInfoData(int taskId, String srcTabName, boolean isTimeOut, String errorMessage) {
        String result = "预处理任务[" + taskId + "," + srcTabName + "] " + (isTimeOut ? "超时!!!!" : "正常") + " "
                + (errorMessage != null ? "错误:[" + errorMessage + "]" : "");
        return result;
    }





}
